﻿using System.Linq;
using System.Windows.Media;
using Clio.Common;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Interfaces;
using System;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;

namespace Follower
{
    public class Follower : IBotPlugin
    {
        public bool Equals(IBotPlugin other)
        {
            throw new NotImplementedException();
        }

        public void OnButtonPress()
        {
            if (Core.Me.CurrentTarget as Character != null && (Core.Me.CurrentTarget as Character).IsAlive && !(Core.Me.CurrentTarget as Character).CanAttack)
            {
                Leader = Core.Me.CurrentTarget;
                Properties.Settings.Default.LeaderName = Leader.Name;
                Logging.Write(Colors.SkyBlue, "[Follower] Setting Leader");
            }
            else if (NewLeader())
            {
                Properties.Settings.Default.LeaderName = Leader.Name;
            }
            else
            {
                Leader = null;
                Properties.Settings.Default.LeaderName = "(null)";
                Logging.Write(Colors.SkyBlue, "[Follower] Clearing Leader");
            }

            if (_form == null || _form.IsDisposed || _form.Disposing)
                _form = new FollowerForm();

            _form.ShowDialog();
        }

        public void OnInitialize()
        {
        }

        public void OnShutdown()
        {
            throw new NotImplementedException();
        }

        public void OnEnabled()
        {
            if (Navigator.NavigationProvider == null)
            {
                Navigator.NavigationProvider = new GaiaNavigator();
            }
        }

        public void OnDisabled()
        {
        }

        public string Author { get { return "kaihaider"; } }
        public Version Version { get { return new Version(0, 3); } }
        public string Name { get { return "Follower"; } }
        public string Description { get { return "Press Config Button to Follow Target"; } }
        public bool WantButton { get { return true; } }
        public string ButtonText { get { return "Follow Target"; } }
        private FollowerForm _form;
        public static GameObject Leader;
        public static MoveResult Mr;
        public static bool WeMoving;
        public static bool LogNoParty;
        public static IPlayerMover MyMover = new SlideMover();
        public static bool IsFacingMovement(GameObject face, GameObject location)
        {
            if (face == null || location == null) return false;
            float rotationToLocation = MathEx.NormalizeRadian(MathHelper.CalculateHeading(face.Location, location.Location) + (float)Math.PI);
            float d = Math.Abs(MathEx.NormalizeRadian(face.Heading - rotationToLocation));
            if (d > Math.PI) d = Math.Abs(d - 2 * (float)Math.PI);
            //Logging.Write("Differance: " + d);
            return d < 0.12;
        }
        public static bool IsFacing(GameObject face, GameObject location)
        {
            if (face == null || location == null) return false;
            float rotationToLocation = MathEx.NormalizeRadian(MathHelper.CalculateHeading(face.Location, location.Location) + (float)Math.PI);
            float d = Math.Abs(MathEx.NormalizeRadian(face.Heading - rotationToLocation));
            if (d > Math.PI) d = Math.Abs(d - 2 * (float)Math.PI);
            //Logging.Write("Differance: " + d);
            return d < 0.785f;
        }

        public static void MoveFailed()
        {
            if (PartyManager.IsInParty)
            {
                if (NewLeader())
                {
                    Mr = Navigator.MoveTo(Leader.Location);
                }
            }
            if (Mr != MoveResult.PathGenerationFailed && Mr != MoveResult.Failed && Mr != MoveResult.Failure) return;
            Logging.Write(Colors.SkyBlue, "[Follower] Mesh failed twice! Manually moving to Leader");
            MyMover.MoveTowards(Leader.Location);
        }

        public static bool NewLeader()
        {
            if (!PartyManager.IsInParty|| PartyManager.VisibleMembers == null)
            {
                if(LogNoParty)
                Logging.Write(Colors.SkyBlue, "[Follower] No Party Found");
                LogNoParty = false;
                return false;
            }
            LogNoParty = true;
            var party = PartyManager.VisibleMembers.Where(
                unit => unit != null && unit.IsValid && !unit.IsMe && (unit.GameObject as Character) != null
                        && (Leader == null || unit.ObjectId != Leader.ObjectId) && !(unit.GameObject as Character).IsNpc &&
                        (unit.GameObject as Character).IsAlive).ToArray();
            var newLeader = party.OrderBy(r => r.GameObject.DistanceSqr(Core.Player.Location)).FirstOrDefault();
            if (newLeader == null) return false;
            Leader = newLeader.GameObject;
            Logging.Write(Colors.SkyBlue, "[Follower] Switching leader with closest party member");
            return true;
        }
        public void OnPulse()
        {
            if (!Core.IsInGame || Core.Me.IsDead || Core.Me.IsCasting || (Leader as Character == null || !Leader.IsValid) && !NewLeader()) return;
            if (Leader.Distance() > 50)
            {
                Leader = null;
                return;
            }
            if (Leader as Character != null && (Leader as Character).TargetCharacter != null && (Leader as Character).TargetCharacter.CanAttack
                && (Leader as Character).TargetCharacter.IsAlive && Core.Me.CurrentTarget != (Leader as Character).TargetCharacter
              && (Leader as Character).InCombat && (Leader as Character).TargetCharacter.InCombat 
              && (Poi.Current == null || Poi.Current.Type == PoiType.None || Poi.Current.Type == PoiType.Wait))
                Poi.Current = new Poi((Leader as Character).TargetCharacter,PoiType.Kill);
            if (Core.Me.InCombat || Poi.Current != null && (Poi.Current.Type == PoiType.Wait || Poi.Current.Type == PoiType.Kill)) return;
            if (!MovementManager.IsMoving && (Leader.Distance(Core.Me.Location) > Convert.ToByte(Properties.Settings.Default.MaxDistance) || !Leader.InLineOfSight()))
            {
                Logging.Write(Colors.SkyBlue, "[Follower] Following Leader");
                if (!Properties.Settings.Default.ChatMove)
                {
                    if (!Properties.Settings.Default.ForceNoMesh)
                    {
                        Mr = Navigator.MoveTo(Leader.Location);
                        if (Mr == MoveResult.PathGenerationFailed || Mr == MoveResult.Failed || Mr == MoveResult.Failure)
                            MoveFailed();
                    }
                    else MyMover.MoveTowards(Leader.Location);
                }
                else
                {
                    Leader.Target();
                    ChatManager.SendChat("/follow");
                }
                WeMoving = true;
            }
            if (MovementManager.IsMoving && Leader.Distance(Core.Me.Location) > Convert.ToByte(Properties.Settings.Default.MaxDistance) && !IsFacing(Core.Me, Leader))
            {
                Logging.Write(Colors.SkyBlue, "[Follower] Why you leaving leader?");
                if (!Properties.Settings.Default.ChatMove)
                {
                    if (!Properties.Settings.Default.ForceNoMesh)
                    {
                        Mr = Navigator.MoveTo(Leader.Location);
                        if (Mr == MoveResult.PathGenerationFailed || Mr == MoveResult.Failed || Mr == MoveResult.Failure)
                            MoveFailed();
                    }
                    else MyMover.MoveTowards(Leader.Location);
                }
                else
                {
                    Leader.Target();
                    ChatManager.SendChat("/follow");
                }
                WeMoving = true;
            }

            if (WeMoving && MovementManager.IsMoving && Leader.InLineOfSight() && Leader.Distance(Core.Me.Location) < Convert.ToByte(Properties.Settings.Default.StopDistance))
            {
                Logging.Write(Colors.SkyBlue, "[Follower] We reached our leader!");
                MyMover.MoveStop();
                WeMoving = false;
            }
        }
    }
}
